"""Oxo GraphQL queries and mutations."""

import ipaddress
import pathlib
import threading
import uuid
from typing import Optional, List

import graphene
import graphql
import httpx
from graphene_file_upload import scalars
from graphql.execution import base as graphql_base

from ostorlab import configuration_manager
from ostorlab import exceptions
from ostorlab.assets import android_aab as android_aab_asset
from ostorlab.assets import android_apk as android_apk_asset
from ostorlab.assets import android_store as android_store_asset
from ostorlab.assets import asset as ostorlab_asset
from ostorlab.assets import domain_name as domain_name_asset
from ostorlab.assets import ios_ipa as ios_ipa_asset
from ostorlab.assets import ios_store as ios_store_asset
from ostorlab.assets import ipv4 as ipv4_address_asset
from ostorlab.assets import ipv6 as ipv6_address_asset
from ostorlab.assets import link as link_asset
from ostorlab.cli import agent_fetcher, install_agent
from ostorlab.runtimes import definitions
from ostorlab.runtimes.local import runtime
from ostorlab.runtimes.local import runtime as local_runtime
from ostorlab.runtimes.local.models import models
from ostorlab.serve_app import common, export_utils
from ostorlab.serve_app import import_utils
from ostorlab.serve_app import types
from ostorlab.utils import defintions as utils_definitions

DEFAULT_NUMBER_ELEMENTS = 15


class Query(graphene.ObjectType):
    """Query object type."""

    scans = graphene.Field(
        types.OxoScansType,
        scan_ids=graphene.List(graphene.Int, required=False),
        page=graphene.Int(required=False),
        number_elements=graphene.Int(required=False),
        order_by=types.OxoScanOrderByEnum(required=False),
        sort=common.SortEnum(required=False),
        description="List of scans.",
    )
    scan = graphene.Field(
        types.OxoScanType, scan_id=graphene.Int(), description="Retrieve scan by id."
    )

    agent_groups = graphene.Field(
        types.OxoAgentGroupsType,
        search=graphene.String(required=False),
        page=graphene.Int(required=False),
        number_elements=graphene.Int(required=False),
        order_by=graphene.Argument(types.AgentGroupOrderByEnum, required=False),
        sort=graphene.Argument(common.SortEnum, required=False),
        agent_group_ids=graphene.List(graphene.Int),
        asset_type=graphene.Argument(types.OxoAssetTypeEnum, required=False),
    )

    def resolve_scans(
        self,
        info: graphql_base.ResolveInfo,
        scan_ids: Optional[List[int]] = None,
        page: Optional[int] = None,
        number_elements: int = DEFAULT_NUMBER_ELEMENTS,
        order_by: Optional[types.OxoScanOrderByEnum] = None,
        sort: Optional[common.SortEnum] = None,
    ) -> Optional[types.OxoScansType]:
        """Resolve scans query.

        Args:
            info: GraphQL resolve info.
            scan_ids: List of scan ids. Defaults to None.
            page: Page number. Defaults to None.
            number_elements: Number of elements. Defaults to DEFAULT_NUMBER_ELEMENTS.
            order_by: Order by filter. Defaults to None.
            sort: Sort filter. Defaults to None.

        Returns:
            Optional[types.OxoScansType]: List of scans.
        """
        if number_elements <= 0:
            return types.OxoScansType(scans=[])

        with models.Database() as session:
            scans = session.query(models.Scan)

            if scan_ids is not None:
                scans = scans.filter(models.Scan.id.in_(scan_ids))

            order_by_filter = None
            if order_by == types.OxoScanOrderByEnum.ScanId:
                order_by_filter = models.Scan.id
            elif order_by == types.OxoScanOrderByEnum.Title:
                order_by_filter = models.Scan.title
            elif order_by == types.OxoScanOrderByEnum.CreatedTime:
                order_by_filter = models.Scan.created_time
            elif order_by == types.OxoScanOrderByEnum.Progress:
                order_by_filter = models.Scan.progress
            if order_by_filter is not None and sort == common.SortEnum.Desc:
                scans = scans.order_by(order_by_filter.desc())
            elif order_by_filter is not None:
                scans = scans.order_by(order_by_filter)
            else:
                scans = scans.order_by(models.Scan.id.desc())

            if page is not None and number_elements > 0:
                p = common.Paginator(scans, number_elements)
                page = p.get_page(page)
                page_info = common.PageInfo(
                    count=p.count,
                    num_pages=p.num_pages,
                    has_next=page.has_next(),
                    has_previous=page.has_previous(),
                )
                return types.OxoScansType(scans=page, page_info=page_info)
            else:
                return types.OxoScansType(scans=scans)

    def resolve_scan(
        self, info: graphql_base.ResolveInfo, scan_id: int
    ) -> types.OxoScanType:
        """Retrieve scan by its id.

        Args:
            info: `graphql_base.ResolveInfo` instance.
            scan_id: The scan ID.

        Raises:
            graphql.GraphQLError in case the scan does not exist.

        Returns:
            The scan information.
        """
        with models.Database() as session:
            scan = session.query(models.Scan).get(scan_id)
            if scan is None:
                raise graphql.GraphQLError("Scan not found.")

            return scan

    def resolve_agent_groups(
        self,
        info,
        search: str = None,
        page=None,
        number_elements: int = DEFAULT_NUMBER_ELEMENTS,
        order_by: Optional[types.AgentGroupOrderByEnum] = None,
        sort: Optional[common.SortEnum] = None,
        agent_group_ids: Optional[List[int]] = None,
        asset_type: Optional[types.OxoAssetTypeEnum] = None,
    ) -> types.OxoAgentGroupsType:
        """Resolve agent groups query.

        Args:
            info: GraphQL resolve info.
            search: Search string.
            page: Page number.
            number_elements: Number of elements.
            order_by: Order by filter.
            sort: Sort filter.
            agent_group_ids: List of agent group ids.

        Returns:
            types.OxoAgentGroupsType: List of agent groups.
        """

        if number_elements <= 0:
            return types.OxoAgentGroupsType(agent_groups=[])

        with models.Database() as session:
            agent_groups_query = session.query(models.AgentGroup)

            if agent_group_ids is not None:
                agent_groups_query = agent_groups_query.filter(
                    models.AgentGroup.id.in_(agent_group_ids)
                )

            if search is not None:
                agent_groups_query = agent_groups_query.filter(
                    models.AgentGroup.name.ilike(f"%{search}%")
                )

            if asset_type is not None:
                asset_type_enum = types.OxoAssetTypeEnum.get(asset_type.upper())
                agent_groups_query = agent_groups_query.join(
                    models.AgentGroup.asset_types
                ).filter_by(type=asset_type_enum)

            order_by_filter = None
            if order_by == types.AgentGroupOrderByEnum.AgentGroupId:
                order_by_filter = models.AgentGroup.id
            elif order_by == types.AgentGroupOrderByEnum.Name:
                order_by_filter = models.AgentGroup.name
            elif order_by == types.AgentGroupOrderByEnum.CreatedTime:
                order_by_filter = models.AgentGroup.created_time

            if sort == common.SortEnum.Desc and order_by_filter is not None:
                agent_groups_query = agent_groups_query.order_by(order_by_filter.desc())
            elif order_by_filter is not None:
                agent_groups_query = agent_groups_query.order_by(order_by_filter)
            else:
                agent_groups_query = agent_groups_query.order_by(
                    models.AgentGroup.id.desc()
                )

            if page is not None and number_elements > 0:
                p = common.Paginator(agent_groups_query, number_elements)
                page = p.get_page(page)
                page_info = common.PageInfo(
                    count=p.count,
                    num_pages=p.num_pages,
                    has_next=page.has_next(),
                    has_previous=page.has_previous(),
                )
                return types.OxoAgentGroupsType(agent_groups=page, page_info=page_info)
            else:
                return types.OxoAgentGroupsType(agent_groups=agent_groups_query)


class ImportScanMutation(graphene.Mutation):
    """Import scan mutation."""

    class Arguments:
        scan_id = graphene.Int(required=False)
        file = scalars.Upload(required=True)

    message = graphene.String()

    @staticmethod
    def mutate(
        root,
        info: graphql_base.ResolveInfo,
        file: scalars.Upload,
        scan_id: Optional[int] = None,
    ) -> "ImportScanMutation":
        """Import scan mutation.

        Args:
            info (graphql_base.ResolveInfo): GraphQL resolve info.
            file (scalars.Upload): File to import.
            scan_id (Optional[int], optional): Scan id. Defaults to None.

        Returns:
            ImportScanMutation: Import scan mutation.
        """
        with models.Database() as session:
            scan = session.query(models.Scan).filter_by(id=scan_id).first()
            import_utils.import_scan(file.read(), scan)
            return ImportScanMutation(message="Scan imported successfully")


class ExportScanMutation(graphene.Mutation):
    """Export scan mutation."""

    class Arguments:
        scan_id = graphene.Int(required=True)
        export_ide = graphene.Boolean(required=False)

    content = common.Bytes()

    @staticmethod
    def mutate(
        root,
        info: graphql_base.ResolveInfo,
        scan_id: int,
        export_ide: Optional[bool] = False,
    ) -> "ExportScanMutation":
        """Export scan mutation.

        Args:
            info: GraphQL resolve info.
            scan_id: Scan id.
            export_ide: Export to IDE. Defaults to False.

        Returns:
            ExportScanMutation: Export scan mutation.
        """
        with models.Database() as session:
            scan = session.query(models.Scan).filter_by(id=scan_id).first()
            if scan is None:
                raise graphql.GraphQLError("Scan not found.")

            export_file_content = export_utils.export_scan(scan, export_ide)
            return ExportScanMutation(content=export_file_content)


class DeleteScansMutation(graphene.Mutation):
    """Delete Scan & its information mutation."""

    class Arguments:
        scan_ids = graphene.List(graphene.Int, required=True)

    result = graphene.Boolean()

    @staticmethod
    def mutate(
        root,
        info: graphql_base.ResolveInfo,
        scan_ids: list[int],
    ) -> "DeleteScansMutation":
        """Delete a scan & its information.

        Args:
            info: `graphql_base.ResolveInfo` instance.
            scan_ids: The scan IDs.

        Raises:
            graphql.GraphQLError in case the scan does not exist.

        Returns:
            Boolean `True` if the delete operation is successful.

        """
        with models.Database() as session:
            scans = (
                session.query(models.Scan).filter(models.Scan.id.in_(scan_ids)).all()
            )
            if len(scans) == 0:
                raise graphql.GraphQLError("No scan is found.")

            for scan in scans:
                scan_query = session.query(models.Scan).filter_by(id=scan.id)
                scan_query.delete()
                session.query(models.Vulnerability).filter_by(scan_id=scan.id).delete()
                session.query(models.ScanStatus).filter_by(scan_id=scan.id).delete()
                DeleteScansMutation.delete_assets(scan.id, session)
                session.commit()
            return DeleteScansMutation(result=True)

    @staticmethod
    def delete_assets(scan_id: int, session: models.Database) -> None:
        """Delete assets.

        Args:
            scan_id: The scan ID.
            session: The database session.
        """

        assets = session.query(models.Asset).filter_by(scan_id=scan_id).all()
        session.query(models.Asset).filter_by(scan_id=scan_id).delete()
        for asset in assets:
            asset_type = asset.type
            if asset_type == "android_file":
                session.query(models.AndroidFile).filter_by(id=asset.id).delete()
            elif asset_type == "ios_file":
                session.query(models.IosFile).filter_by(id=asset.id).delete()
            elif asset_type == "android_store":
                session.query(models.AndroidStore).filter_by(id=asset.id).delete()
            elif asset_type == "ios_store":
                session.query(models.IosStore).filter_by(id=asset.id).delete()
            elif asset_type == "network":
                session.query(models.Network).filter_by(id=asset.id).delete()
                session.query(models.IPRange).filter_by(
                    network_asset_id=asset.id
                ).delete()
            elif asset_type == "urls":
                session.query(models.Urls).filter_by(id=asset.id).delete()
                session.query(models.Link).filter_by(urls_asset_id=asset.id).delete()
            elif asset_type == "domain_asset":
                session.query(models.DomainAsset).filter_by(id=asset.id).delete()
                session.query(models.DomainName).filter_by(
                    domain_asset_id=asset.id
                ).delete()


class CreateAssetsMutation(graphene.Mutation):
    """Create asset mutation."""

    class Arguments:
        assets = graphene.List(types.OxoAssetInputType, required=True)

    assets = graphene.List(types.OxoAssetType)

    @staticmethod
    def mutate(
        root, info: graphql_base.ResolveInfo, assets: List[types.OxoAssetInputType]
    ):
        """Create asset mutation."""
        created_assets = []
        errors = []
        config_manager = configuration_manager.ConfigurationManager()
        for asset in assets:
            error_message = CreateAssetsMutation._validate(asset)
            if error_message is not None:
                errors.append(error_message)
                continue
            if asset.android_store is not None:
                for asset_android_store in asset.android_store:
                    new_asset = models.AndroidStore.create(
                        package_name=asset_android_store.package_name,
                        application_name=asset_android_store.application_name,
                    )
                    created_assets.append(new_asset)
            if asset.android_apk_file is not None:
                for asset_android_apk_file in asset.android_apk_file:
                    content = asset_android_apk_file.file.read()
                    android_file_path = (
                        config_manager.upload_path / f"android_{str(uuid.uuid4())}"
                    )
                    android_file_path.write_bytes(content)
                    new_asset = models.AndroidFile.create(
                        package_name=asset_android_apk_file.package_name,
                        path=str(android_file_path),
                    )
                    created_assets.append(new_asset)
            if asset.android_aab_file is not None:
                for asset_android_aab_file in asset.android_aab_file:
                    content = asset_android_aab_file.file.read()
                    android_file_path = (
                        config_manager.upload_path / f"android_{str(uuid.uuid4())}"
                    )
                    android_file_path.write_bytes(content)
                    new_asset = models.AndroidFile.create(
                        package_name=asset_android_aab_file.package_name,
                        path=str(android_file_path),
                    )
                    created_assets.append(new_asset)
            if asset.ios_store is not None:
                for asset_ios_store in asset.ios_store:
                    new_asset = models.IosStore.create(
                        bundle_id=asset_ios_store.bundle_id,
                        application_name=asset_ios_store.application_name,
                    )
                    created_assets.append(new_asset)
            if asset.ios_file is not None:
                for asset_ios_file in asset.ios_file:
                    content = asset_ios_file.file.read()
                    ios_file_path = (
                        config_manager.upload_path / f"ios_{str(uuid.uuid4())}"
                    )
                    ios_file_path.write_bytes(content)
                    new_asset = models.IosFile.create(
                        bundle_id=asset_ios_file.bundle_id,
                        path=str(ios_file_path),
                    )
                    created_assets.append(new_asset)
            if asset.link is not None:
                new_asset = models.Urls.create(links=asset.link)
                created_assets.append(new_asset)
            if asset.ip is not None:
                new_asset = models.Network.create(networks=asset.ip)
                created_assets.append(new_asset)
            if asset.domain is not None:
                new_asset = models.DomainAsset.create(domains=asset.domain)
                created_assets.append(new_asset)
        if len(errors) > 0:
            error_messages = "\n".join(errors)
            raise graphql.GraphQLError(f"Invalid assets: {error_messages}")

        return CreateAssetsMutation(assets=created_assets)

    @staticmethod
    def _validate(asset: types.OxoAssetInputType) -> Optional[str]:
        """Validate asset API input & return corresponding error message."""
        assets = []
        if asset.android_store is not None:
            assets.append(asset.android_store)
        if asset.android_apk_file is not None:
            assets.append(asset.android_apk_file)
        if asset.android_aab_file is not None:
            assets.append(asset.android_aab_file)
        if asset.ios_store is not None:
            assets.append(asset.ios_store)
        if asset.ios_file is not None:
            assets.append(asset.ios_file)
        if asset.link is not None:
            assets.append(asset.link)
        if asset.ip is not None:
            assets.append(asset.ip)
        if asset.domain is not None:
            assets.append(asset.domain)

        if len(assets) == 0:
            return f"Asset {asset} input is missing target."
        elif len(assets) >= 2:
            return f"Single target input must be defined for asset {asset}."
        else:
            return None


class StopScansMutation(graphene.Mutation):
    """Stop scan mutation."""

    class Arguments:
        scan_ids = graphene.List(graphene.Int, required=True)

    scans = graphene.List(types.OxoScanType)

    @staticmethod
    def mutate(root, info: graphql_base.ResolveInfo, scan_ids: list[int]):
        """Stop the desired scan.

        Args:
            info: `graphql_base.ResolveInfo` instance.
            scan_ids: The scan IDs.

        Raises:
            graphql.GraphQLError in case the scan does not exist or the scan id is invalid.

        Returns:
            The stopped scan.

        """
        with models.Database() as session:
            scans = (
                session.query(models.Scan).filter(models.Scan.id.in_(scan_ids)).all()
            )
            if len(scans) == 0:
                raise graphql.GraphQLError("No scan is found.")
            for scan_id in scan_ids:
                local_runtime.LocalRuntime().stop(scan_id=str(scan_id))
            return StopScansMutation(scans=scans)


class PublishAgentGroupMutation(graphene.Mutation):
    """Create agent group."""

    class Arguments:
        agent_group = types.OxoAgentGroupCreateInputType(required=True)

    agent_group = graphene.Field(types.OxoAgentGroupType)

    @staticmethod
    def mutate(
        root,
        info: graphql_base.ResolveInfo,
        agent_group: types.OxoAgentGroupCreateInputType,
    ) -> "PublishAgentGroupMutation":
        """Create agent group.

        Args:
            info (graphql_base.ResolveInfo): GraphQL resolve info.
            agent_group (types.OxoAgentGroupCreateInputType): Agent group to create.

        Returns:
            PublishAgentGroupMutation: Publish agent group mutation.
        """
        asset_types = [
            types.OxoAssetTypeEnum.get(asset_type.upper())
            for asset_type in agent_group.asset_types
        ]
        group = models.AgentGroup.create(
            name=agent_group.name,
            description=agent_group.description,
            agents=agent_group.agents,
            asset_types=asset_types,
        )
        return PublishAgentGroupMutation(agent_group=group)


class DeleteAgentGroupMutation(graphene.Mutation):
    """Delete agent group mutation."""

    class Arguments:
        agent_group_id = graphene.Int(required=True)

    result = graphene.Boolean()

    @staticmethod
    def mutate(
        root,
        info: graphql_base.ResolveInfo,
        agent_group_id: int,
    ) -> "DeleteAgentGroupMutation":
        """Delete agent group mutation.

        Args:
            info (graphql_base.ResolveInfo): GraphQL resolve info.
            agent_group_id (int): Agent group id.

        Returns:
            DeleteAgentGroupMutation: Delete agent group mutation.
        """
        with models.Database() as session:
            agent_group_query = session.query(models.AgentGroup).filter_by(
                id=agent_group_id
            )
            if agent_group_query.count() == 0:
                raise graphql.GraphQLError("AgentGroup not found.")
            mappings = session.query(models.AgentGroupMapping).filter_by(
                agent_group_id=agent_group_id
            )
            if mappings.count() > 0:
                mappings.delete()

            agent_group_query.delete()

            session.commit()
            return DeleteAgentGroupMutation(result=True)


class RunScanMutation(graphene.Mutation):
    class Arguments:
        scan = types.OxoAgentScanInputType(required=True)

    scan = graphene.Field(types.OxoScanType)

    @staticmethod
    def _prepare_agent_group(agent_group_id: int) -> definitions.AgentGroupDefinition:
        """Prepare agent group.

        Args:
            agent_group_id: The agent group id.

        Returns:
            The agent group.
        """
        with models.Database() as session:
            agent_group = (
                session.query(models.AgentGroup).filter_by(id=agent_group_id).first()
            )

            if agent_group is None:
                raise graphql.GraphQLError("Agent group not found.")

            agent_group_instance = definitions.AgentGroupDefinition(
                name=agent_group.name,
                description=agent_group.description,
                agents=[
                    definitions.AgentSettings(
                        key=agent.key.split(":")[0],
                        version=agent.key.split(":")[1] if ":" in agent.key else None,
                        args=[
                            utils_definitions.Arg.build(
                                name=arg.name,
                                type=arg.type,
                                value=arg.value,
                                description=arg.description,
                            )
                            for arg in session.query(models.AgentArgument)
                            .filter_by(agent_id=agent.id)
                            .all()
                        ],
                    )
                    for agent in agent_group.agents
                ],
            )
            return agent_group_instance

    @staticmethod
    def _prepare_assets(asset_ids: List[int]) -> List[ostorlab_asset.Asset]:
        """Prepare assets.

        Args:
            asset_ids: The asset ids.

        Returns:
            The assets.
        """

        with models.Database() as session:
            assets = (
                session.query(models.Asset).filter(models.Asset.id.in_(asset_ids)).all()
            )

            if assets is None or len(assets) == 0:
                raise graphql.GraphQLError("Assets not found.")

            scan_assets = []
            for asset in assets:
                if asset.type == "android_file":
                    file_path = pathlib.Path(asset.path)
                    if file_path.exists() is False:
                        raise graphql.GraphQLError(f"File {asset.path} not found.")
                    file_bytes = file_path.read_bytes()
                    if (
                        common.is_apk(file_bytes) is True
                        or common.is_xapk(file_bytes) is True
                    ):
                        scan_assets.append(
                            android_apk_asset.AndroidApk(
                                content=file_bytes, path=asset.path
                            )
                        )
                    elif common.is_aab(file_bytes) is True:
                        scan_assets.append(
                            android_aab_asset.AndroidAab(
                                content=file_bytes, path=asset.path
                            )
                        )
                    else:
                        raise graphql.GraphQLError(
                            f"Unsupported file type: {asset.path}"
                        )
                elif asset.type == "ios_file":
                    file_path = pathlib.Path(asset.path)
                    if file_path.exists() is False:
                        raise graphql.GraphQLError(f"File {asset.path} not found.")

                    scan_assets.append(
                        ios_ipa_asset.IOSIpa(
                            content=file_path.read_bytes(), path=asset.path
                        )
                    )
                elif asset.type == "android_store":
                    scan_assets.append(
                        android_store_asset.AndroidStore(
                            package_name=asset.package_name
                        )
                    )
                elif asset.type == "ios_store":
                    scan_assets.append(
                        ios_store_asset.IOSStore(bundle_id=asset.bundle_id)
                    )
                elif asset.type == "network":
                    ips = (
                        session.query(models.IPRange)
                        .filter_by(network_asset_id=asset.id)
                        .all()
                    )
                    for ip in ips:
                        ip_network = ipaddress.ip_network(ip.host, strict=False)
                        if ip_network.version == 4:
                            scan_assets.append(
                                ipv4_address_asset.IPv4(
                                    host=ip_network.network_address.exploded,
                                    mask=str(ip.mask)
                                    if ip.mask is not None
                                    else str(ip_network.prefixlen),
                                )
                            )
                        elif ip_network.version == 6:
                            scan_assets.append(
                                ipv6_address_asset.IPv6(
                                    host=ip_network.network_address.exploded,
                                    mask=str(ip.mask)
                                    if ip.mask is not None
                                    else str(ip_network.prefixlen),
                                )
                            )
                        else:
                            raise graphql.GraphQLError(f"Invalid IP address {ip}")
                elif asset.type == "urls":
                    links = (
                        session.query(models.Link)
                        .filter_by(urls_asset_id=asset.id)
                        .all()
                    )
                    for link in links:
                        scan_assets.append(
                            link_asset.Link(url=link.url, method=link.method)
                        )
                elif asset.type == "domain_asset":
                    domains = (
                        session.query(models.DomainName)
                        .filter_by(domain_asset_id=asset.id)
                        .all()
                    )
                    for domain in domains:
                        scan_assets.append(
                            domain_name_asset.DomainName(name=domain.name)
                        )
                else:
                    raise graphql.GraphQLError("Unsupported asset type.")

            return scan_assets

    @staticmethod
    def _install_agents(
        agent_group: definitions.AgentGroupDefinition,
        runtime_instance: local_runtime.LocalRuntime,
    ) -> None:
        """Install agents.

        Args:
            agent_group: The agent group.
            runtime_instance: The runtime instance.
        """

        try:
            runtime_instance.install()
            for ag in agent_group.agents:
                try:
                    install_agent.install(ag.key, ag.version)
                except agent_fetcher.AgentDetailsNotFound:
                    graphql.GraphQLError(f"Agent {ag.key} not found on the store.")
        except httpx.HTTPError as e:
            raise graphql.GraphQLError(f"Could not install the agents: {e}")

    @staticmethod
    def mutate(
        root,
        info: graphql_base.ResolveInfo,
        scan: types.OxoAgentScanInputType,
    ) -> "RunScanMutation":
        """Run scan mutation.

        Args:
            info: `graphql_base.ResolveInfo` instance.
            scan: The scan information.

        Raises:
            graphql.GraphQLError in case of an error.

        Returns:
            The scan information.
        """

        agent_group = RunScanMutation._prepare_agent_group(scan.agent_group_id)
        scan_assets = RunScanMutation._prepare_assets(scan.asset_ids)

        runtime_instance: runtime.LocalRuntime = runtime.LocalRuntime()
        runtime_instance.follow = []
        created_scan = runtime_instance.prepare_scan(
            assets=scan_assets, title=scan.title
        )
        RunScanMutation._persist_scan_info(created_scan, scan)

        thread = threading.Thread(
            target=RunScanMutation._run_scan_background,
            args=(runtime_instance, agent_group, scan, scan_assets),
        )
        thread.start()
        return RunScanMutation(scan=created_scan)

    @staticmethod
    def _persist_scan_info(
        created_scan: models.Scan,
        scan: types.OxoAgentScanInputType,
    ) -> None:
        """Persist scan infos related to agent group and assets.

        Args:
            created_scan: The created scan.
            scan: The scan information.
        """
        with models.Database() as session:
            created_scan.agent_group_id = scan.agent_group_id
            session.add(created_scan)
            session.commit()
            assets_db = session.query(models.Asset).filter(
                models.Asset.id.in_(scan.asset_ids)
            )

            for asset in assets_db:
                asset.scan_id = created_scan.id

            session.commit()

    @staticmethod
    def _run_scan_background(
        runtime_instance: runtime.LocalRuntime,
        agent_group: definitions.AgentGroupDefinition,
        scan: types.OxoAgentScanInputType,
        scan_assets: List[ostorlab_asset.Asset],
    ) -> None:
        """Run scan in background.

        Args:
            runtime_instance: The runtime instance.
            agent_group: The agent group.
            scan: The scan information.
            scan_assets: The scan assets.
        """

        try:
            can_run_scan = runtime_instance.can_run(agent_group_definition=agent_group)
        except exceptions.OstorlabError as e:
            raise graphql.GraphQLError(f"Runtime encountered an error to run scan: {e}")

        if can_run_scan is True:
            RunScanMutation._install_agents(agent_group, runtime_instance)
            try:
                runtime_instance.scan(
                    title=scan.title,
                    agent_group_definition=agent_group,
                    assets=scan_assets,
                )

            except exceptions.OstorlabError as e:
                raise graphql.GraphQLError(
                    f"Runtime encountered an error to run scan: {e}"
                )


class Mutations(graphene.ObjectType):
    delete_scans = DeleteScansMutation.Field(
        description="Delete a scan & all its information."
    )
    delete_agent_group = DeleteAgentGroupMutation.Field(
        description="Delete agent group."
    )
    import_scan = ImportScanMutation.Field(description="Import scan from file.")
    export_scan = ExportScanMutation.Field(description="Export scan to file.")
    create_assets = CreateAssetsMutation.Field(description="Create an asset.")
    stop_scans = StopScansMutation.Field(
        description="Stops running scan, scan is marked as stopped once the engine has completed cancellation."
    )
    publish_agent_group = PublishAgentGroupMutation.Field(
        description="Create agent group"
    )
    run_scan = RunScanMutation.Field(description="Run scan")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ostorlab/agent/message/proto/v3/capture/http/response/response.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nDostorlab/agent/message/proto/v3/capture/http/response/response.proto\x12\x35ostorlab.agent.message.proto.v3.capture.http.response\"%\n\x06header\x12\x0c\n\x04name\x18\x01 \x01(\x0c\x12\r\n\x05value\x18\x02 \x01(\x0c\"\xd7\x01\n\x07Message\x12\n\n\x02id\x18\x01 \x01(\t\x12\x13\n\x0bstatus_code\x18\x02 \x01(\x05\x12N\n\x07headers\x18\x03 \x03(\x0b\x32=.ostorlab.agent.message.proto.v3.capture.http.response.header\x12\x0f\n\x07\x63ontent\x18\x04 \x01(\x0c\x12\x14\n\x0chttp_version\x18\x05 \x01(\x0c\x12\x0e\n\x06reason\x18\x06 \x01(\t\x12\x12\n\ntime_start\x18\x07 \x01(\x02\x12\x10\n\x08time_end\x18\x08 \x01(\x02')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ostorlab.agent.message.proto.v3.capture.http.response.response_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _HEADER._serialized_start=127
  _HEADER._serialized_end=164
  _MESSAGE._serialized_start=167
  _MESSAGE._serialized_end=382
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ostorlab/scanner/proto/assets/network.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ostorlab.scanner.proto.assets import ip_pb2 as ostorlab_dot_scanner_dot_proto_dot_assets_dot_ip__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+ostorlab/scanner/proto/assets/network.proto\x12%ostorlab.scanner.proto.assets.network\x1a&ostorlab/scanner/proto/assets/ip.proto\"A\n\x07Message\x12\x36\n\x03ips\x18\x01 \x03(\x0b\x32).ostorlab.scanner.proto.assets.ip.Message')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ostorlab.scanner.proto.assets.network_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _MESSAGE._serialized_start=126
  _MESSAGE._serialized_end=191
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ostorlab/agent/message/proto/v3/report/vulnerability/vulnerability.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ostorlab.agent.message.proto.v3.asset.domain_name import domain_name_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_domain__name_dot_domain__name__pb2
from ostorlab.agent.message.proto.v3.asset.ip.v4 import v4_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_ip_dot_v4_dot_v4__pb2
from ostorlab.agent.message.proto.v3.asset.ip.v6 import v6_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_ip_dot_v6_dot_v6__pb2
from ostorlab.agent.message.proto.v3.asset.link import link_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_link_dot_link__pb2
from ostorlab.agent.message.proto.v3.asset.store.android_store import android_store_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_store_dot_android__store_dot_android__store__pb2
from ostorlab.agent.message.proto.v3.asset.store.ios_store import ios_store_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_store_dot_ios__store_dot_ios__store__pb2
from ostorlab.agent.message.proto.v3.asset.file import file_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_file__pb2
from ostorlab.agent.message.proto.v3.asset.file.android.aab import aab_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_android_dot_aab_dot_aab__pb2
from ostorlab.agent.message.proto.v3.asset.file.android.apk import apk_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_android_dot_apk_dot_apk__pb2
from ostorlab.agent.message.proto.v3.asset.file.ios.ipa import ipa_pb2 as ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_ios_dot_ipa_dot_ipa__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='ostorlab/agent/message/proto/v3/report/vulnerability/vulnerability.proto',
  package='ostorlab.agent.message.proto.v3.report.vulnerability',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\nHostorlab/agent/message/proto/v3/report/vulnerability/vulnerability.proto\x12\x34ostorlab.agent.message.proto.v3.report.vulnerability\x1a\x43ostorlab/agent/message/proto/v3/asset/domain_name/domain_name.proto\x1a\x34ostorlab/agent/message/proto/v3/asset/ip/v4/v4.proto\x1a\x34ostorlab/agent/message/proto/v3/asset/ip/v6/v6.proto\x1a\x35ostorlab/agent/message/proto/v3/asset/link/link.proto\x1aMostorlab/agent/message/proto/v3/asset/store/android_store/android_store.proto\x1a\x45ostorlab/agent/message/proto/v3/asset/store/ios_store/ios_store.proto\x1a\x35ostorlab/agent/message/proto/v3/asset/file/file.proto\x1a@ostorlab/agent/message/proto/v3/asset/file/android/aab/aab.proto\x1a@ostorlab/agent/message/proto/v3/asset/file/android/apk/apk.proto\x1a<ostorlab/agent/message/proto/v3/asset/file/ios/ipa/ipa.proto\"\'\n\tReference\x12\r\n\x05title\x18\x01 \x01(\t\x12\x0b\n\x03url\x18\x02 \x01(\t\"o\n\x08Metadata\x12\r\n\x05value\x18\x01 \x01(\t\x12T\n\x04type\x18\x03 \x01(\x0e\x32\x46.ostorlab.agent.message.proto.v3.report.vulnerability.MetadataTypeEnum\"\x8d\x07\n\x15VulnerabilityLocation\x12Q\n\x0b\x64omain_name\x18\x01 \x01(\x0b\x32:.ostorlab.agent.message.proto.v3.asset.domain_name.MessageH\x00\x12\x44\n\x04ipv4\x18\x02 \x01(\x0b\x32\x34.ostorlab.agent.message.proto.v3.asset.ip.v4.MessageH\x00\x12\x44\n\x04ipv6\x18\x03 \x01(\x0b\x32\x34.ostorlab.agent.message.proto.v3.asset.ip.v6.MessageH\x00\x12\x43\n\x04link\x18\x04 \x01(\x0b\x32\x33.ostorlab.agent.message.proto.v3.asset.link.MessageH\x00\x12[\n\randroid_store\x18\x05 \x01(\x0b\x32\x42.ostorlab.agent.message.proto.v3.asset.store.android_store.MessageH\x00\x12S\n\tios_store\x18\x06 \x01(\x0b\x32>.ostorlab.agent.message.proto.v3.asset.store.ios_store.MessageH\x00\x12\x43\n\x04\x66ile\x18\x07 \x01(\x0b\x32\x33.ostorlab.agent.message.proto.v3.asset.file.MessageH\x00\x12V\n\x0b\x61ndroid_aab\x18\x08 \x01(\x0b\x32?.ostorlab.agent.message.proto.v3.asset.file.android.aab.MessageH\x00\x12V\n\x0b\x61ndroid_apk\x18\t \x01(\x0b\x32?.ostorlab.agent.message.proto.v3.asset.file.android.apk.MessageH\x00\x12N\n\x07ios_ipa\x18\n \x01(\x0b\x32;.ostorlab.agent.message.proto.v3.asset.file.ios.ipa.MessageH\x00\x12P\n\x08metadata\x18\x64 \x03(\x0b\x32>.ostorlab.agent.message.proto.v3.report.vulnerability.MetadataB\x07\n\x05\x61sset\"0\n\rCategoryGroup\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x12\n\ncategories\x18\x02 \x03(\t\"\x82\x06\n\x07Message\x12\r\n\x05title\x18\x01 \x01(\t\x12U\n\x0brisk_rating\x18\x02 \x01(\x0e\x32@.ostorlab.agent.message.proto.v3.report.vulnerability.RiskRating\x12\x18\n\x10technical_detail\x18\x03 \x01(\t\x12\x19\n\x11short_description\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12\x16\n\x0erecommendation\x18\x06 \x01(\t\x12S\n\nreferences\x18\x07 \x03(\x0b\x32?.ostorlab.agent.message.proto.v3.report.vulnerability.Reference\x12\x1d\n\x0esecurity_issue\x18\x08 \x01(\x08:\x05\x66\x61lse\x12\x1c\n\rprivacy_issue\x18\t \x01(\x08:\x05\x66\x61lse\x12!\n\x12has_public_exploit\x18\n \x01(\x08:\x05\x66\x61lse\x12\"\n\x13targeted_by_malware\x18\x0b \x01(\x08:\x05\x66\x61lse\x12%\n\x16targeted_by_ransomware\x18\x0c \x01(\x08:\x05\x66\x61lse\x12\'\n\x18targeted_by_nation_state\x18\r \x01(\x08:\x05\x66\x61lse\x12\x0b\n\x03\x64na\x18\x0e \x01(\t\x12\x16\n\x0e\x63vss_v3_vector\x18\x0f \x01(\t\x12k\n\x16vulnerability_location\x18\x10 \x01(\x0b\x32K.ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation\x12\\\n\x0f\x63\x61tegory_groups\x18\x11 \x03(\x0b\x32\x43.ostorlab.agent.message.proto.v3.report.vulnerability.CategoryGroup\x12\x16\n\x0e\x63vss_v4_vector\x18\x12 \x01(\t*~\n\nRiskRating\x12\x08\n\x04HIGH\x10\x00\x12\n\n\x06MEDIUM\x10\x01\x12\x07\n\x03LOW\x10\x02\x12\x0f\n\x0bPOTENTIALLY\x10\x03\x12\r\n\tHARDENING\x10\x04\x12\n\n\x06SECURE\x10\x05\x12\r\n\tIMPORTANT\x10\x06\x12\x08\n\x04INFO\x10\x07\x12\x0c\n\x08\x43RITICAL\x10\x08*P\n\x10MetadataTypeEnum\x12\r\n\tFILE_PATH\x10\x00\x12\x11\n\rCODE_LOCATION\x10\x01\x12\x07\n\x03URL\x10\x02\x12\x08\n\x04PORT\x10\x03\x12\x07\n\x03LOG\x10\x04'
  ,
  dependencies=[ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_domain__name_dot_domain__name__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_ip_dot_v4_dot_v4__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_ip_dot_v6_dot_v6__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_link_dot_link__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_store_dot_android__store_dot_android__store__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_store_dot_ios__store_dot_ios__store__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_file__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_android_dot_aab_dot_aab__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_android_dot_apk_dot_apk__pb2.DESCRIPTOR,ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_ios_dot_ipa_dot_ipa__pb2.DESCRIPTOR,])

_RISKRATING = _descriptor.EnumDescriptor(
  name='RiskRating',
  full_name='ostorlab.agent.message.proto.v3.report.vulnerability.RiskRating',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='HIGH', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MEDIUM', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOW', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='POTENTIALLY', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='HARDENING', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SECURE', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='IMPORTANT', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INFO', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CRITICAL', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2650,
  serialized_end=2776,
)
_sym_db.RegisterEnumDescriptor(_RISKRATING)

RiskRating = enum_type_wrapper.EnumTypeWrapper(_RISKRATING)
_METADATATYPEENUM = _descriptor.EnumDescriptor(
  name='MetadataTypeEnum',
  full_name='ostorlab.agent.message.proto.v3.report.vulnerability.MetadataTypeEnum',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='FILE_PATH', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CODE_LOCATION', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='URL', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PORT', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOG', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2778,
  serialized_end=2858,
)
_sym_db.RegisterEnumDescriptor(_METADATATYPEENUM)

MetadataTypeEnum = enum_type_wrapper.EnumTypeWrapper(_METADATATYPEENUM)
HIGH = 0
MEDIUM = 1
LOW = 2
POTENTIALLY = 3
HARDENING = 4
SECURE = 5
IMPORTANT = 6
INFO = 7
CRITICAL = 8
FILE_PATH = 0
CODE_LOCATION = 1
URL = 2
PORT = 3
LOG = 4



_REFERENCE = _descriptor.Descriptor(
  name='Reference',
  full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Reference',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='title', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Reference.title', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='url', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Reference.url', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=761,
  serialized_end=800,
)


_METADATA = _descriptor.Descriptor(
  name='Metadata',
  full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Metadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Metadata.value', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Metadata.type', index=1,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=802,
  serialized_end=913,
)


_VULNERABILITYLOCATION = _descriptor.Descriptor(
  name='VulnerabilityLocation',
  full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='domain_name', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.domain_name', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ipv4', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.ipv4', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ipv6', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.ipv6', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='link', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.link', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='android_store', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.android_store', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ios_store', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.ios_store', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='file', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.file', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='android_aab', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.android_aab', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='android_apk', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.android_apk', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ios_ipa', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.ios_ipa', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.metadata', index=10,
      number=100, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='asset', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation.asset',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=916,
  serialized_end=1825,
)


_CATEGORYGROUP = _descriptor.Descriptor(
  name='CategoryGroup',
  full_name='ostorlab.agent.message.proto.v3.report.vulnerability.CategoryGroup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.CategoryGroup.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='categories', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.CategoryGroup.categories', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1827,
  serialized_end=1875,
)


_MESSAGE = _descriptor.Descriptor(
  name='Message',
  full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='title', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.title', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='risk_rating', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.risk_rating', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='technical_detail', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.technical_detail', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='short_description', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.short_description', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.description', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='recommendation', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.recommendation', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='references', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.references', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='security_issue', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.security_issue', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='privacy_issue', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.privacy_issue', index=8,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='has_public_exploit', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.has_public_exploit', index=9,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='targeted_by_malware', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.targeted_by_malware', index=10,
      number=11, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='targeted_by_ransomware', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.targeted_by_ransomware', index=11,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='targeted_by_nation_state', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.targeted_by_nation_state', index=12,
      number=13, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dna', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.dna', index=13,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cvss_v3_vector', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.cvss_v3_vector', index=14,
      number=15, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='vulnerability_location', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.vulnerability_location', index=15,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='category_groups', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.category_groups', index=16,
      number=17, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cvss_v4_vector', full_name='ostorlab.agent.message.proto.v3.report.vulnerability.Message.cvss_v4_vector', index=17,
      number=18, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1878,
  serialized_end=2648,
)

_METADATA.fields_by_name['type'].enum_type = _METADATATYPEENUM
_VULNERABILITYLOCATION.fields_by_name['domain_name'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_domain__name_dot_domain__name__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['ipv4'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_ip_dot_v4_dot_v4__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['ipv6'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_ip_dot_v6_dot_v6__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['link'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_link_dot_link__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['android_store'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_store_dot_android__store_dot_android__store__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['ios_store'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_store_dot_ios__store_dot_ios__store__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['file'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_file__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['android_aab'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_android_dot_aab_dot_aab__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['android_apk'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_android_dot_apk_dot_apk__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['ios_ipa'].message_type = ostorlab_dot_agent_dot_message_dot_proto_dot_v3_dot_asset_dot_file_dot_ios_dot_ipa_dot_ipa__pb2._MESSAGE
_VULNERABILITYLOCATION.fields_by_name['metadata'].message_type = _METADATA
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['domain_name'])
_VULNERABILITYLOCATION.fields_by_name['domain_name'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['ipv4'])
_VULNERABILITYLOCATION.fields_by_name['ipv4'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['ipv6'])
_VULNERABILITYLOCATION.fields_by_name['ipv6'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['link'])
_VULNERABILITYLOCATION.fields_by_name['link'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['android_store'])
_VULNERABILITYLOCATION.fields_by_name['android_store'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['ios_store'])
_VULNERABILITYLOCATION.fields_by_name['ios_store'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['file'])
_VULNERABILITYLOCATION.fields_by_name['file'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['android_aab'])
_VULNERABILITYLOCATION.fields_by_name['android_aab'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['android_apk'])
_VULNERABILITYLOCATION.fields_by_name['android_apk'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_VULNERABILITYLOCATION.oneofs_by_name['asset'].fields.append(
  _VULNERABILITYLOCATION.fields_by_name['ios_ipa'])
_VULNERABILITYLOCATION.fields_by_name['ios_ipa'].containing_oneof = _VULNERABILITYLOCATION.oneofs_by_name['asset']
_MESSAGE.fields_by_name['risk_rating'].enum_type = _RISKRATING
_MESSAGE.fields_by_name['references'].message_type = _REFERENCE
_MESSAGE.fields_by_name['vulnerability_location'].message_type = _VULNERABILITYLOCATION
_MESSAGE.fields_by_name['category_groups'].message_type = _CATEGORYGROUP
DESCRIPTOR.message_types_by_name['Reference'] = _REFERENCE
DESCRIPTOR.message_types_by_name['Metadata'] = _METADATA
DESCRIPTOR.message_types_by_name['VulnerabilityLocation'] = _VULNERABILITYLOCATION
DESCRIPTOR.message_types_by_name['CategoryGroup'] = _CATEGORYGROUP
DESCRIPTOR.message_types_by_name['Message'] = _MESSAGE
DESCRIPTOR.enum_types_by_name['RiskRating'] = _RISKRATING
DESCRIPTOR.enum_types_by_name['MetadataTypeEnum'] = _METADATATYPEENUM
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Reference = _reflection.GeneratedProtocolMessageType('Reference', (_message.Message,), {
  'DESCRIPTOR' : _REFERENCE,
  '__module__' : 'ostorlab.agent.message.proto.v3.report.vulnerability.vulnerability_pb2'
  # @@protoc_insertion_point(class_scope:ostorlab.agent.message.proto.v3.report.vulnerability.Reference)
  })
_sym_db.RegisterMessage(Reference)

Metadata = _reflection.GeneratedProtocolMessageType('Metadata', (_message.Message,), {
  'DESCRIPTOR' : _METADATA,
  '__module__' : 'ostorlab.agent.message.proto.v3.report.vulnerability.vulnerability_pb2'
  # @@protoc_insertion_point(class_scope:ostorlab.agent.message.proto.v3.report.vulnerability.Metadata)
  })
_sym_db.RegisterMessage(Metadata)

VulnerabilityLocation = _reflection.GeneratedProtocolMessageType('VulnerabilityLocation', (_message.Message,), {
  'DESCRIPTOR' : _VULNERABILITYLOCATION,
  '__module__' : 'ostorlab.agent.message.proto.v3.report.vulnerability.vulnerability_pb2'
  # @@protoc_insertion_point(class_scope:ostorlab.agent.message.proto.v3.report.vulnerability.VulnerabilityLocation)
  })
_sym_db.RegisterMessage(VulnerabilityLocation)

CategoryGroup = _reflection.GeneratedProtocolMessageType('CategoryGroup', (_message.Message,), {
  'DESCRIPTOR' : _CATEGORYGROUP,
  '__module__' : 'ostorlab.agent.message.proto.v3.report.vulnerability.vulnerability_pb2'
  # @@protoc_insertion_point(class_scope:ostorlab.agent.message.proto.v3.report.vulnerability.CategoryGroup)
  })
_sym_db.RegisterMessage(CategoryGroup)

Message = _reflection.GeneratedProtocolMessageType('Message', (_message.Message,), {
  'DESCRIPTOR' : _MESSAGE,
  '__module__' : 'ostorlab.agent.message.proto.v3.report.vulnerability.vulnerability_pb2'
  # @@protoc_insertion_point(class_scope:ostorlab.agent.message.proto.v3.report.vulnerability.Message)
  })
_sym_db.RegisterMessage(Message)


# @@protoc_insertion_point(module_scope)



"""Unittests for KB generator."""

import os.path
from pytest_mock import plugin
from pyfakefs import fake_filesystem

from tests import utils
from tools import kb_generator


def testGenerateKB_whenVulnerabilityProvided_returnsKBEntry(
    mocker: plugin.MockerFixture,
) -> None:
    """generate_kb is responsible for generating KB entries from a vulnerability name.
    when provided with a valid vulnerability name, this function should return a dict
    representing a KB entry
    """
    mocker.patch(
        "openai.api_resources.chat_completion.ChatCompletion.create",
        side_effect=utils.mock_chat_completion_create,
    )
    vulnerability = kb_generator.Vulnerability(
        "Sql Injection", kb_generator.RiskRating.HIGH, kb_generator.Platform.WEB
    )

    kbentry = kb_generator.generate_kb(vulnerability)

    assert (
        "SQL injection is a type of security vulnerability that allows an attacker to inject malicious SQL "
        "statements into an application's database" in kbentry.description
    )
    assert "import 'package:sqflite/sqflite.dart';" in kbentry.description
    assert (
        "it is important to use parameterized queries or prepared statements instead of concatenating user input "
        "directly into SQL statements. " in kbentry.recommendation
    )
    assert kbentry.meta["risk_rating"] == "high"
    assert (
        kbentry.meta["short_description"]
        == "SQL Injection vulnerability allows an attacker to execute malicious SQL "
        "queries to the database,potentially gaining access to sensitive data or "
        "performing unauthorized actions."
    )
    assert kbentry.meta["references"] == {
        "NIST": "https://nvd.nist.gov/vuln/detail/CVE-2019-16759",
        "OWASP ": "https: //owasp.org/www-community/attacks/SQL_Injection",
        "SANS": "https://www.sans.org/top-25-software-errors/#cat3",
    }
    assert kbentry.meta["title"] == "SQL Injection Vulnerability"
    assert kbentry.meta["security_issue"] is True
    assert kbentry.meta["privacy_issue"] is False
    assert kbentry.meta["categories"] == {
        "OWASP_MASVS_L1": [
            "V2: Authentication and Session Management",
            "V3: Cryptography",
            "V4: Network Communication",
            "V5: Platform Interaction",
            "V6: Code Quality and Build Settings",
        ],
        "OWASP_MASVS_L2": ["V7: Data Protection", "V8: Resilience Against Attack"],
    }


def testDumpKB_whenPathIsValid_writesFiles(
    mocker: plugin.MockerFixture, fs: fake_filesystem.FakeFilesystem
) -> None:
    """Test creation of KB files."""
    vulnerability = kb_generator.Vulnerability(
        name="XSS",
        platform=kb_generator.Platform.WEB,
        risk_rating=kb_generator.RiskRating.HIGH,
    )
    kbentry = kb_generator.KBEntry(
        vulnerability=vulnerability,
        description="Description",
        recommendation="Recommendation",
        meta={"Meta": "Meta data"},
    )
    kb_generator.dump_kb(kbentry)

    assert os.path.exists("WEB_SERVICE/WEB/_HIGH") is True
    assert os.path.exists("WEB_SERVICE/WEB/_HIGH/description.md") is True
    assert os.path.exists("WEB_SERVICE/WEB/_HIGH/recommendation.md") is True
    assert os.path.exists("WEB_SERVICE/WEB/_HIGH/meta.json") is True



"""Correct asset-scan relationship

Revision ID: e0ef8ab7d038
Revises: 736027d2cc2f
Create Date: 2024-06-06 09:55:52.686173

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "e0ef8ab7d038"
down_revision = "736027d2cc2f"
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("asset", schema=None) as batch_op:
        batch_op.add_column(sa.Column("scan_id", sa.Integer(), nullable=True))
        batch_op.create_foreign_key(
            batch_op.f("fk_asset_scan_id_scan"), "scan", ["scan_id"], ["id"]
        )

    with op.batch_alter_table("scan", schema=None) as batch_op:
        batch_op.drop_constraint("fk_scan_asset_id_asset", type_="foreignkey")
        batch_op.drop_column("asset_id")

    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("scan", schema=None) as batch_op:
        batch_op.add_column(sa.Column("asset_id", sa.INTEGER(), nullable=True))
        batch_op.create_foreign_key(
            "fk_scan_asset_id_asset", "asset", ["asset_id"], ["id"]
        )

    with op.batch_alter_table("asset", schema=None) as batch_op:
        batch_op.drop_constraint(
            batch_op.f("fk_asset_scan_id_scan"), type_="foreignkey"
        )
        batch_op.drop_column("scan_id")

    # ### end Alembic commands ###



"""import utils module: contains utility functions to import the scan details."""

import datetime
import io
import ipaddress
import json
import uuid
import zipfile
from typing import Optional

from ostorlab import configuration_manager
from ostorlab.runtimes.local.models import models, utils
from ostorlab.utils import risk_rating

SCAN_JSON = "scan.json"
ASSET_JSON = "asset.json"
VULNERABILITY_JSON = "vulnerability.json"


def import_scan(
    file_data: bytes,
    append_to_scan: Optional[models.Scan] = None,
) -> None:
    """Import the scan details from the given file data.

    Args:
        file_data (bytes): The file data to import.
        append_to_scan (Optional[models.Scan], optional): The scan to append to. Defaults to None.
    """
    file = io.BytesIO(file_data)
    scan = append_to_scan or models.Scan()
    with zipfile.ZipFile(file, "r", zipfile.ZIP_DEFLATED, True) as archive:
        _import_scan(scan, archive)
        _import_vulnz(scan, archive)


def _import_scan(scan: models.Scan, archive: zipfile.ZipFile) -> None:
    with models.Database() as session:
        scan_dict = json.loads(archive.read(SCAN_JSON))
        asset_dicts = (
            [
                json.loads(asset_dict)
                for asset_dict in archive.read(ASSET_JSON).strip().split(b"\n")
            ]
            if ASSET_JSON in archive.namelist()
            else []
        )
        scan.title = scan_dict.get("title")
        scan.created_time = (
            datetime.datetime.strptime(
                scan_dict.get("created_time"), "%Y-%m-%d %H:%M:%S"
            )
            if scan_dict.get("created_time") is not None
            else datetime.datetime.now()
        )
        last_status: Optional[str] = None
        if scan.id is None:
            session.add(scan)

        for status in sorted(scan_dict["status"], key=lambda s: s["id"]):
            scan_id: int = scan.id
            models.ScanStatus.create(
                key=status["key"], value=status["value"], scan_id=scan_id
            )
            if status["key"] == "progress":
                last_status = status["value"].upper()
        scan.progress = models.ScanProgress[last_status]
        scan.risk_rating = (
            risk_rating.RiskRating[scan_dict.get("risk_rating").upper()]
            if scan_dict.get("risk_rating") is not None
            else None
        )
        session.merge(scan)
        session.commit()

        _import_asset(scan.id, asset_dicts, archive)


def _get_asset_str(asset_dicts: list) -> str:
    """Get the asset string from the given asset dictionaries.

    Args:
        asset_dicts: The asset dictionaries.

    Returns:
        The asset string.
    """

    asset_str = []
    for asset_dict in asset_dicts:
        if "android file" in asset_dict["type"].lower():
            asset_str.append("Android File")
        elif "ios file" in asset_dict["type"].lower():
            asset_str.append("IOS File")
        elif "url" in asset_dict["type"].lower():
            urls_str = ", ".join(asset_dict.get("urls"))
            asset_str.append(f"Url(s): {urls_str}")
        elif "network" in asset_dict["type"].lower():
            networks_str = ", ".join(asset_dict.get("networks"))
            asset_str.append(f"Network(s): {networks_str}")
        elif "android store" in asset_dict["type"].lower():
            asset_str.append(
                f"Android Store: {asset_dict.get('application_name') or asset_dict.get('package_name')}"
            )
        elif "ios store" in asset_dict["type"].lower():
            asset_str.append(
                f"IOS Store: {asset_dict.get('application_name') or asset_dict.get('package_name')}"
            )

    return ", ".join(asset_str)


def _import_vulnz(scan: models.Scan, archive: zipfile.ZipFile) -> None:
    vulnerabilities = json.loads(archive.read(VULNERABILITY_JSON))
    for vulnerability in vulnerabilities:
        models.Vulnerability.create(
            technical_detail=vulnerability.get("technical_detail"),
            risk_rating=vulnerability.get("risk_rating").upper(),
            title=vulnerability.get("detail").get("title"),
            short_description=vulnerability.get("detail").get("short_description"),
            description=vulnerability.get("detail").get("description"),
            recommendation=vulnerability.get("detail").get("recommendation"),
            scan_id=scan.id,
            references=vulnerability.get("detail").get("references"),
            location=vulnerability.get("detail").get("location"),
            cvss_v3_vector=vulnerability.get("cvss_v3_vector"),
        )


def _import_asset(
    scan_id: int, asset_dicts: list[dict[str, str]], archive: zipfile.ZipFile
) -> None:
    config_manager = configuration_manager.ConfigurationManager()

    for asset_dict in asset_dicts:
        if "android file" in asset_dict["type"].lower():
            if asset_dict["path"] in archive.namelist():
                content = archive.read(asset_dict["path"])
                android_file_path = (
                    config_manager.upload_path / f"android_{str(uuid.uuid4())}"
                )
                android_file_path.write_bytes(content)
                models.AndroidFile.create(
                    package_name=utils.get_package_name(str(android_file_path)),
                    path=str(android_file_path),
                    scan_id=scan_id,
                )

        elif "ios file" in asset_dict["type"].lower():
            if asset_dict["path"] in archive.namelist():
                content = archive.read(asset_dict["path"])
                ios_file_path = config_manager.upload_path / f"ios_{str(uuid.uuid4())}"
                ios_file_path.write_bytes(content)
                models.IosFile.create(
                    bundle_id=utils.get_bundle_id(str(ios_file_path)),
                    path=str(ios_file_path),
                    scan_id=scan_id,
                )

        elif "url" in asset_dict["type"].lower():
            urls = []
            for url in asset_dict["urls"]:
                urls.append({"url": url, "method": "GET"})
            models.Urls.create(
                links=urls,
                scan_id=scan_id,
            )

        elif "network" in asset_dict["type"].lower():
            networks = []
            for network in asset_dict["networks"]:
                ip_network = ipaddress.ip_network(network, strict=False)
                networks.append(
                    {
                        "host": ip_network.network_address.exploded,
                        "mask": str(ip_network.prefixlen),
                    }
                )
            models.Network.create(
                networks=networks,
                scan_id=scan_id,
            )

        elif "android store" in asset_dict["type"].lower():
            application_name = asset_dict["application_name"]
            package_name = asset_dict["package_name"]
            models.AndroidStore.create(
                application_name=application_name,
                package_name=package_name,
                scan_id=scan_id,
            )

        elif "ios store" in asset_dict["type"].lower():
            application_name = asset_dict["application_name"]
            bundle_id = asset_dict["package_name"]
            models.IosStore.create(
                application_name=application_name,
                bundle_id=bundle_id,
                scan_id=scan_id,
            )

        else:
            raise NotImplementedError(
                f"Asset type {asset_dict['type']} not implemented."
            )



"""Receive scanner config (Nats creds, container registry creds ...)."""

from typing import Dict, Optional
import json

from ostorlab.apis import request


class ScannerConfigAPIRequest(request.APIRequest):
    """Get scanner config."""

    def __init__(self, scanner_id: str):
        self._scanner_id = scanner_id

    @property
    def query(self) -> Optional[str]:
        """Defines the query to get the configs.

        Returns:
            The query to get the configs.
        """
        return """
         query scanners($scannerUuid: UUID!){
          scanners(scannerUuid: $scannerUuid){
            scanners{
              id
              name
              description
              config{
                registryConfiguration{
                  accountName
                  credentials
                  url
                }
                busUrl
                busClusterId
                busClientName
                subjectBusConfigs{
                    subjectBusConfigs{
                        subject
                        queue
                    }
                }
              }
            }
          }
        }
        """

    @property
    def data(self) -> Optional[Dict]:
        """Sets the query to get the configs.

        Returns:
              The query to get the configs.
        """
        data = {
            "query": self.query,
            "variables": json.dumps({"scannerUuid": self._scanner_id}),
        }
        return data



"""Android store package target asset."""

import dataclasses

from ostorlab.assets import asset


@dataclasses.dataclass
@asset.selector("v3.asset.store.android_store")
class AndroidStore(asset.Asset):
    """Android store package target asset."""

    def __init__(self, package_name: str):
        self.package_name = package_name

    def __str__(self) -> str:
        return f"Android Store: ({self.package_name})"

    @property
    def proto_field(self) -> str:
        return "android_store"



"""This module contains custom types for the CLI commands."""

import dataclasses
from typing import Optional

import click

OSTORLAB_AGENTS_PREFIX = "agent/ostorlab"


@dataclasses.dataclass
class AgentArg:
    """Dataclass representing an agent argument passed from cli."""

    name: str
    value: str


class AgentArgType(click.ParamType):
    """Custom Click type for parsing agent arguments in the format 'arg_name:arg_value'."""

    name = "arg_name:arg_value"

    def convert(
        self,
        arg_value: str,
        param: Optional[click.Parameter],
        ctx: Optional[click.Context],
    ) -> AgentArg:
        """Override convert method from ParamType class to parse command line arguments of agents in the format
        'arg_name:arg_value' and convert them into AgentArg objects.

        Args:
            arg_value: The argument value in the format `arg_name:arg_value`.
            param: The parameter using this type for conversion. Defaults to None.
            ctx: The current click context. Defaults to None.

        Returns:
            AgentArg: An AgentArg object representing the parsed argument.

        Raises:
            click.BadParameter: If the argument value cannot be parsed into the expected format.
        """
        try:
            arg_name, arg_value = arg_value.split(":")
            return AgentArg(name=arg_name, value=arg_value)
        except ValueError:
            self.fail(
                message=f"Invalid argument {arg_value}. The expected format is name:value. Example: --arg "
                f"fast_mode:false",
                param=param,
                ctx=ctx,
            )


class AgentKeyType(click.ParamType):
    """Custom Click type for parsing agent keys."""

    name = "agent_key"

    def convert(
        self,
        cli_value: str,
        param: Optional[click.Parameter],
        ctx: Optional[click.Context],
    ) -> str:
        """Override convert method from ParamType class to parse agent keys and convert them into the right key if
        applicable.

        Args:
            cli_value: The cli value for agent key.
            param: The parameter using this type for conversion. Defaults to None.
            ctx: The current click context. Defaults to None.

        Returns:
            str: The agent key.

        Raises:
            click.BadParameter: If the agent key is malformed.
        """
        forward_slash_count = cli_value.count("/")
        if forward_slash_count == 0:
            return f"{OSTORLAB_AGENTS_PREFIX}/{cli_value}"
        elif forward_slash_count == 1 and cli_value.startswith("@") is True:
            return f"agent/{cli_value[1:]}"
        elif forward_slash_count == 2:
            return cli_value
        else:
            self.fail(
                message=f"Invalid agent key: {cli_value}. The expected formats are:  (agent/<org>/<name> or "
                f"@<org>/<name>). Org name can be omitted for defaults agent hosted by Ostorlab.",
                param=param,
                ctx=ctx,
            )



"""This module triggers a scan using the android_store asset."""

import logging
from typing import Tuple, Optional

import click

from ostorlab.cli.scan.run import run
from ostorlab.cli import console as cli_console
from ostorlab.assets import android_store as android_store_asset
from ostorlab import exceptions

console = cli_console.Console()
logger = logging.getLogger(__name__)


@run.run.command(name="android-store")
@click.option("--package-name", multiple=True, required=False)
@click.pass_context
def android_store(
    ctx: click.core.Context, package_name: Optional[Tuple[str]] = ()
) -> None:
    """Run scan for a package_name."""
    runtime = ctx.obj["runtime"]
    assets = []
    if package_name != ():
        for package in package_name:
            assets.append(android_store_asset.AndroidStore(package_name=package))
    else:
        console.error("Command missing a package name.")
        raise click.exceptions.Exit(2)

    logger.debug("scanning assets %s", [str(asset) for asset in assets])
    try:
        created_scan = runtime.scan(
            title=ctx.obj["title"],
            agent_group_definition=ctx.obj["agent_group_definition"],
            assets=assets,
        )
        if created_scan is not None:
            runtime.link_agent_group_scan(
                created_scan, ctx.obj["agent_group_definition"]
            )
            runtime.link_assets_scan(created_scan.id, assets)

    except exceptions.OstorlabError as e:
        console.error(f"An error was encountered while running the scan: {e}")



"""RabbitMQ service in charge of routing Agent messages."""

import binascii
import hashlib
import logging
import os
import pathlib
import uuid
from typing import Dict

import docker
import tenacity
from docker import errors
from docker import types
from docker.models import services

logger = logging.getLogger(__name__)

MQ_IMAGE = "rabbitmq:3.9-management"
MQ_ADVANCED_CONF_PATH = "/etc/rabbitmq/advanced.config"


class LocalRabbitMQ:
    """RabbitMQ service spawned a docker swarm service."""

    def __init__(
        self,
        name: str,
        network: str,
        exposed_ports: Dict[int, int] = None,
        image: str = MQ_IMAGE,
    ) -> None:
        """Initialize the MQ service parameters.
        Args:
            name: Name of the service.
            network: Network used for the Docker MQ service.
            exposed_ports: The list of MQ service exposed ports
            image: MQ Docker image
        """
        self._uuid = uuid.uuid4()
        self._name = name
        self._docker_client = docker.from_env()
        # images
        self._mq_image = image
        self._network = network
        self._mq_host = f"mq_{self._name}"
        # service
        self._mq_service = None
        # exposed_port
        self._exposed_ports = exposed_ports

    @property
    def url(self) -> str:
        """URL to connect to the local RabbitMQ instance."""
        return f"amqp://guest:guest@{self._mq_host}:5672/"

    @property
    def vhost(self):
        """Default vhost."""
        return "/"

    @property
    def service(self):
        """The RabbitMQ corresponding docker service."""
        return self._mq_service

    @property
    def management_url(self) -> str:
        """URL to connect to the management interface of the RabbitMQ instance."""
        return f"http://guest:guest@{self._mq_host}:15672/"

    def start(self) -> None:
        """Start local Rabbit MQ instance."""
        self._create_network()
        self._mq_service = self._start_mq()

    def stop(self):
        """Stop local Rabiit MQ instance."""
        for service in self._docker_client.services.list():
            universe = service.attrs["Spec"]["Labels"].get("ostorlab.universe")
            if (
                universe is not None
                and service.name.startswith("mq_")
                and self._name in universe
            ):
                service.remove()

    def _create_network(self):
        if any(
            network.name == self._network
            for network in self._docker_client.networks.list()
        ):
            logger.warning("network already exists.")
        else:
            logger.info("creating private network %s", self._network)
            return self._docker_client.networks.create(
                name=self._network,
                driver="overlay",
                attachable=True,
                labels={"ostorlab.universe": self._name},
                check_duplicate=True,
            )

    def _create_mq_advanced_config(self) -> types.services.ConfigReference:
        conf_path = (
            pathlib.Path(__file__).parent / "configurations/mq_advanced_conf.config"
        )
        with open(conf_path, "rb") as conf_file:
            mq_advanced_configuration = conf_file.read()
        config_name = hashlib.md5(
            f"mq_advanced_config_{self._name}_{self._uuid}".encode()
        ).hexdigest()

        try:
            mq_advanced_config = self._docker_client.configs.get(config_name)
            logging.warning(
                "found existing config %s, config will removed", config_name
            )
            mq_advanced_config.remove()
        except docker.errors.NotFound:
            logging.debug("all good, config %s is new", config_name)

        docker_config = self._docker_client.configs.create(
            name=config_name,
            labels={
                "ostorlab.universe": self._name,
                "ostorlab.mq.advanced.config": "true",
            },
            data=mq_advanced_configuration,
        )
        return types.services.ConfigReference(
            config_id=docker_config.id,
            config_name=config_name,
            filename=MQ_ADVANCED_CONF_PATH,
        )

    def _start_mq(self) -> services.Service:
        try:
            logger.info("starting MQ")
            endpoint_spec = types.services.EndpointSpec(
                mode="vip", ports=self._exposed_ports
            )
            service_mode = types.services.ServiceMode("replicated", replicas=1)
            mq_advanced_configuration = self._create_mq_advanced_config()
            configs = [mq_advanced_configuration]
            persistent_storage = docker.types.Mount(
                target="/var/lib/rabbitmq", source=f"{self._name}_mq_data"
            )
            return self._docker_client.services.create(
                image=self._mq_image,
                networks=[self._network],
                hostname="mq",
                name=self._mq_host,
                env=[
                    "TASK_ID={{.Task.Slot}}",
                    f"MQ_SERVICE_NAME={self._mq_host}",
                    f"RABBITMQ_ERLANG_COOKIE={binascii.hexlify(os.urandom(10)).decode()}",
                ],
                restart_policy=types.RestartPolicy(condition="any"),
                mode=service_mode,
                labels={"ostorlab.universe": self._name, "ostorlab.mq": ""},
                configs=configs,
                endpoint_spec=endpoint_spec,
                mounts=[persistent_storage],
            )
        except docker.errors.APIError as e:
            error_message = f"MQ service could not be started. Reason: {e}."
            logger.error(error_message)
            return

    @tenacity.retry(
        stop=tenacity.stop_after_attempt(20),
        wait=tenacity.wait_fixed(0.5),
        # return last value and don't raise RetryError exception.
        retry_error_callback=lambda lv: lv.outcome,
        retry=tenacity.retry_if_result(lambda v: v is False),
    )
    def is_service_healthy(self) -> bool:
        logger.info("checking service %s", self._mq_service.name)
        return self.is_healthy

    @property
    def is_healthy(self) -> bool:
        """Check if the local Rabbit MQ instance is healthy.

        Returns:
            True if the instance is healthy, False otherwise.
        """
        try:
            return (
                self._mq_service is not None
                and len(
                    [
                        task
                        for task in self._mq_service.tasks()
                        if task["Status"]["State"] == "running"
                    ]
                )
                == 1
            )
        except errors.DockerException:
            return False
